from shortuuid.main import decode
from shortuuid.main import encode
from shortuuid.main import get_alphabet
from shortuuid.main import random
from shortuuid.main import set_alphabet
from shortuuid.main import ShortUUID
from shortuuid.main import uuid

__version__ = "1.0.11"
__all__ = [
    "decode",
    "encode",
    "get_alphabet",
    "random",
    "set_alphabet",
    "ShortUUID",
    "uuid",
]



from typing import Any
from typing import Dict
from typing import Tuple

from django.db import models
from django.utils.translation import gettext_lazy as _

from . import ShortUUID


class ShortUUIDField(models.CharField):
    description = _("A short UUID field.")

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        self.length: int = kwargs.pop("length", 22)  # type: ignore
        self.prefix: str = kwargs.pop("prefix", "")  # type: ignore

        if "max_length" not in kwargs:
            # If `max_length` was not specified, set it here.
            kwargs["max_length"] = self.length + len(self.prefix)  # type: ignore

        self.alphabet: str = kwargs.pop("alphabet", None)  # type: ignore
        kwargs["default"] = self._generate_uuid  # type: ignore

        super().__init__(*args, **kwargs)

    def _generate_uuid(self) -> str:
        """Generate a short random string."""
        return self.prefix + ShortUUID(alphabet=self.alphabet).random(
            length=self.length
        )

    def deconstruct(self) -> Tuple[str, str, Tuple, Dict[str, Any]]:
        name, path, args, kwargs = super().deconstruct()
        kwargs["alphabet"] = self.alphabet
        kwargs["length"] = self.length
        kwargs["prefix"] = self.prefix
        kwargs.pop("default", None)
        return name, path, args, kwargs



import argparse
import sys
from typing import Any
from uuid import UUID

from .main import decode
from .main import encode
from .main import uuid


def encode_cli(args: argparse.Namespace):
    print(encode(args.uuid))


def decode_cli(args: argparse.Namespace):
    print(str(decode(args.shortuuid, legacy=args.legacy)))


def cli(*args: Any) -> None:
    parser = argparse.ArgumentParser(
        description="Generate, encode and decode shortuuids",
        epilog="top-level command generates a random shortuuid",
    )

    subparsers = parser.add_subparsers(help="sub-command help")

    encode_parser = subparsers.add_parser(
        "encode", help="Encode a UUID into a short UUID", description=encode.__doc__
    )
    encode_parser.add_argument("uuid", type=UUID, help="UUID to be encoded")
    encode_parser.set_defaults(func=encode_cli)

    decode_parser = subparsers.add_parser(
        "decode", help="Decode a short UUID into a UUID", description=decode.__doc__
    )
    decode_parser.add_argument("shortuuid", type=str, help="Short UUID to be decoded")
    decode_parser.add_argument("--legacy", action="store_true")
    decode_parser.set_defaults(func=decode_cli)

    passed_args = parser.parse_args(*args)

    if hasattr(passed_args, "func"):
        passed_args.func(passed_args)
    else:
        # Maintain legacy behaviour
        print(uuid())


if __name__ == "__main__":
    cli(sys.argv[1:])
